﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using Com.LuisPedroFonseca.ProCamera2D;

public class EventManager : UnitySingleton<EventManager>
{
    public MiniPlayer miniPlayer;
    public Transform upgradeMove;
    public Transform player;
    public AudioSource audioSource;

    void Start()
    {
        ShowEvent1();
    }

    public void ShowEvent1()
    {
        InterfaceManager.instance.ShowIntro();
        SoundManager.PlaySFXLoop(audioSource, "intro");
    }

    public void ShowEvent2()
    {
        StartCoroutine(ShowEvent2CR());
    }

    private IEnumerator ShowEvent2CR()
    {
        InterfaceManager.instance.panelIntro.GetComponent<Animator>().SetTrigger("close");
        SoundManager.CrossOut(2, audioSource);
        yield return new WaitForSeconds(2f);

        ProCamera2D.Instance.AddCameraTarget(miniPlayer.transform);

        InterfaceManager.instance.HideIntro();
        InterfaceManager.instance.HideDialogPanel();

        miniPlayer.GetComponent<Collider2D>().enabled = false;

        yield return new WaitForSeconds(1);

        miniPlayer.GetComponent<Collider2D>().enabled = true;

        yield return new WaitForSeconds(0.1f);

        SoundManager.PlaySFX("fall");
        ProCamera2DShake.Instance.ShakeUsingPreset("Fall");

        yield return new WaitForSeconds(2);

        yield return StartCoroutine(miniPlayer.StandCR());

        InterfaceManager.instance.ShowDialogPanel("Ouch!", ShowEvent3);
    }

    public void ShowEvent3()
    {
        StartCoroutine(ShowEvent3CR());
    }

    private IEnumerator ShowEvent3CR()
    {
        InterfaceManager.instance.HideDialogPanel();

        yield return StartCoroutine(miniPlayer.MoveTo(miniPlayer.transform.position + new Vector3(5, 0, 0)));

        InterfaceManager.instance.ShowDialogPanel("Where am I?", ShowEvent4);
    }

    public void ShowEvent4()
    {
        StartCoroutine(ShowEvent4CR());
    }

    private IEnumerator ShowEvent4CR()
    {
        InterfaceManager.instance.HideDialogPanel();

        yield return StartCoroutine(miniPlayer.MoveTo(miniPlayer.transform.position + new Vector3(-7, 0, 0)));

        InterfaceManager.instance.ShowDialogPanel("There is something glowing up there.", ShowEvent5);

        yield return new WaitForSeconds(0.5f);

        ProCamera2D.Instance.AddCameraTarget(upgradeMove);
    }

    public void ShowEvent5()
    {
        StartCoroutine(ShowEvent5CR());
    }

    private IEnumerator ShowEvent5CR()
    {
        InterfaceManager.instance.HideDialogPanel();

        ProCamera2D.Instance.RemoveCameraTarget(upgradeMove);

        yield return StartCoroutine(miniPlayer.MoveTo(miniPlayer.transform.position + new Vector3(-1, 0, 0)));

        InterfaceManager.instance.ShowDialogPanel("And there is this weird artifact.", ShowEvent6);

        ProCamera2D.Instance.AddCameraTarget(player);
    }

    public void ShowEvent6()
    {
        StartCoroutine(ShowEvent6CR());
    }

    private IEnumerator ShowEvent6CR()
    {
        InterfaceManager.instance.HideDialogPanel();

        ProCamera2D.Instance.RemoveCameraTarget(upgradeMove);

        Vector2 newPos = player.position;
        newPos.y = miniPlayer.transform.position.y;

        yield return StartCoroutine(miniPlayer.MoveTo(newPos));

        InterfaceManager.instance.ShowDialogPanel("There is a panel here. I wonder what will happen if I press this button...", ShowEvent7);
    }

    public void ShowEvent7()
    {
        StartCoroutine(ShowEvent7CR());
    }

    private IEnumerator ShowEvent7CR()
    {
        SoundManager.PlaySFX("door");

        InterfaceManager.instance.HideDialogPanel();

        player.GetComponent<PlayerController>().ShowDoor(true);

        yield return new WaitForSeconds(1f);

        InterfaceManager.instance.ShowDialogPanel("What is inside?", ShowEvent8);
    }

    public void ShowEvent8()
    {
        StartCoroutine(ShowEvent8CR());
    }

    private IEnumerator ShowEvent8CR()
    {
        InterfaceManager.instance.HideDialogPanel();

        ProCamera2D.Instance.RemoveCameraTarget(miniPlayer.transform);

        miniPlayer.gameObject.SetActive(false);

        SoundManager.PlaySFX("door");

        player.Find("Door").GetComponent<Animator>().SetTrigger("close");

        yield return new WaitForSeconds(1f);

        player.GetComponent<PlayerController>().ShowDoor(false);

        InterfaceManager.instance.ShowDialogPanel("There are some buttons in here", ShowEvent9);
    }

    public void ShowEvent9()
    {
        InterfaceManager.instance.HideDialogPanel();
        InterfaceManager.instance.panelButtonsAnimator.SetTrigger("open");
        SoundManager.PlaySFXLoop(audioSource, "mainTheme");
    }

    public void ShowEvent10()
    {
        InterfaceManager.instance.panelButtonsAnimator.SetTrigger("close");
        InterfaceManager.instance.ShowDialogPanel("The button looks different. I wonder what have changed.", ShowEvent11);
    }

    public void ShowEvent11()
    {
        InterfaceManager.instance.HideDialogPanel();
        InterfaceManager.instance.ChangeBurnIcon();
    }

    public void ShowEvent12()
    {
        InterfaceManager.instance.panelButtonsAnimator.SetTrigger("close");
        InterfaceManager.instance.ShowDialogPanel("Everything inside the artifact is glowing now.", ShowEvent13);
    }

    public void ShowEvent13()
    {
        InterfaceManager.instance.HideDialogPanel();
        InterfaceManager.instance.ChangeDashIcon();
    }

    public void ShowEvent14()
    {
        ProCamera2D.Instance.RemoveCameraTarget(player);
        InterfaceManager.instance.panelButtonsAnimator.SetTrigger("close");
        InterfaceManager.instance.ShowDialogPanel("Hey, I can see a light up ahead!", ShowEvent15);
    }

    public void ShowEvent15()
    {
        StartCoroutine(ShowEvent15CR());
    }

    private IEnumerator ShowEvent15CR()
    {
        SoundManager.CrossOut(1, audioSource);

        InterfaceManager.instance.HideDialogPanel();
        InterfaceManager.instance.ShowCredits();

        yield return new WaitForSeconds(1);

        SoundManager.PlaySFXLoop(audioSource, "intro");
    }
}