﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System;

public class InterfaceManager : UnitySingleton<InterfaceManager>
{

    private PlayerController pc;

    public Animator panelButtonsAnimator;

    public GameObject panelDialog;
    public GameObject panelCredits;
    public GameObject panelIntro;
    public Button btnDialog;
    public Text txtDialog;

    public Image dashIcon;
    public Image burnIcon;

    public Sprite dashLevel1;
    public Sprite burnLevel1;

    void Awake()
    {
        base.Awake();

        pc = GameObject.FindObjectOfType<PlayerController>();
    }

    public void OnClickDash()
    {
        pc.dashPressed = true;
    }

    public void OnReleaseDash()
    {
        pc.dashReleased = false;
        pc.dashReleased = true;
    }

    public void OnClickBurn()
    {
        pc.burnPressed = true;
    }

    public void OnReleaseBurn()
    {
        pc.burnReleased = false;
        pc.burnReleased = true;
    }

    public void ChangeDashIcon()
    {
        StartCoroutine("ChangeDashIconCR");
    }

    private IEnumerator ChangeDashIconCR()
    {
        yield return new WaitForSeconds(0.5f);
        dashIcon.sprite = dashLevel1;
        panelButtonsAnimator.SetTrigger("open");
    }

    public void ChangeBurnIcon()
    {
        StartCoroutine("ChangeBurnIconCR");
    }

    private IEnumerator ChangeBurnIconCR()
    {
        yield return new WaitForSeconds(0.5f);
        burnIcon.sprite = burnLevel1;
        panelButtonsAnimator.SetTrigger("open");
    }

    public void ShowDialogPanel(string text, Action clickEvent)
    {
        btnDialog.onClick.RemoveAllListeners();
        btnDialog.onClick.AddListener(() => {
            clickEvent();
            btnDialog.onClick.RemoveAllListeners();
            SoundManager.PlaySFX("beep");
        });
        panelDialog.SetActive(true);
        txtDialog.text = text;
    }

    public void HideDialogPanel()
    {
        panelDialog.SetActive(false);
    }

    public void ShowCredits()
    {
        panelCredits.SetActive(true);
    }

    public void ShowIntro()
    {
        panelIntro.SetActive(true);
    }

    public void HideIntro()
    {
        panelIntro.SetActive(false);
    }
}
