﻿using UnityEngine;
using System.Collections;
using System;

public class PlayerController : MonoBehaviour
{

    public float torqueForce = 500;
    public float burnForce = 8000;
    public float propellerForce = 10000;
    public float maxMomentum = 15000;

    public int moveLevel = 0;
    public int burnLevel = 1;
    public float momentum;
    public float momentumSpeed;

    public GameObject burner;
    public GameObject dashEffect;
    public GameObject door;
    public AudioSource audioSource;

    private Rigidbody2D rigidbody2D;

    public bool dashPressed;
    public bool dashReleased;
    public bool burnPressed;
    public bool burnReleased;

    void Start()
    {
        rigidbody2D = GetComponent<Rigidbody2D>();
    }

    void Update()
    {
        if (dashPressed)
        {
            if (moveLevel == 0)
            {
                AddTorque(torqueForce);
            }
            else
            {
                if (!audioSource.isPlaying)
                {
                    SoundManager.PlaySFXLoop(audioSource, "dash");
                }

                if (momentum > maxMomentum / 2)
                {
                    dashEffect.SetActive(true);
                }
                momentum = Mathf.Clamp(momentum + Time.deltaTime * momentumSpeed, 0, maxMomentum);
                AddTorque(-momentum);
                rigidbody2D.constraints = RigidbodyConstraints2D.FreezePositionX;
            }
        }

        if (dashReleased)
        {
            SoundManager.StopSFXObject(audioSource);

            if (moveLevel == 1)
            {
                SoundManager.PlaySFX(audioSource, "dashEnd");
            }

            dashEffect.SetActive(false);
            AddTorque(-momentum);
            rigidbody2D.constraints = RigidbodyConstraints2D.None;
            momentum = 0;
            dashReleased = false;
            dashPressed = false;
        }

        if (burnPressed)
        {
            if (!audioSource.isPlaying)
            {
                SoundManager.PlaySFXLoop(audioSource, "fire");
            }
            if (burnLevel == 0)
            {
                AddForceUp(burnForce);
            }
            else
            {
                AddForceUp(propellerForce);
            }
        }

        if (burnReleased)
        {
            SoundManager.StopSFXObject(audioSource);
            burner.SetActive(false);
            burnPressed = false;
            burnReleased = false;
        }
    }

    private void AddTorque(float torque)
    {
        rigidbody2D.AddTorque(torque);
        rigidbody2D.AddForce(Vector2.left * (torque / 2), ForceMode2D.Impulse);
    }

    private void AddForceUp(float force)
    {
        rigidbody2D.AddForce(Vector2.up * force);
        burner.SetActive(true);
        if (burnLevel == 1)
        {
            burner.GetComponent<Animator>().SetTrigger("full");
        }
    }

    public void ShowDoor(bool show)
    {
        door.SetActive(show);
    }

    void OnCollisionEnter2D(Collision2D col)
    {
        if (col.relativeVelocity.magnitude > 2)
        {
            SoundManager.PlaySFX("hit");
        }
    }
}